# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-31 20:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("rest_email_auth", "0003_unique_emails")]

    operations = [
        migrations.AddField(
            model_name="emailaddress",
            name="is_primary",
            field=models.BooleanField(
                default=False,
                help_text="Boolean indicating if the email is the user's primary address.",
                verbose_name="is primary",
            ),
        )
    ]
